# lingo - payment and billing system
# Copyright (C) 2023  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import Http404
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.timezone import now
from django.views.generic import FormView, TemplateView, View

from lingo.basket.models import Basket
from lingo.invoicing.views.utils import PDFMixin


class BasketDetailView(LoginRequiredMixin, TemplateView):
    template_name = 'lingo/basket/basket_detail.html'

    def get_context_data(self, **kwargs):
        nameid = self.request.user.get_name_id()
        basket = Basket.objects.filter(
            status__in=['open', 'tobepaid'],
            payer_nameid=nameid,
        ).first()
        kwargs['basket'] = basket
        return super().get_context_data(**kwargs)


basket_detail = BasketDetailView.as_view()


class BasketInvoicePDFView(LoginRequiredMixin, PDFMixin, View):
    def get_object(self):
        nameid = self.request.user.get_name_id()
        basket = get_object_or_404(
            Basket,
            status__in=['open', 'tobepaid'],
            payer_nameid=nameid,
        )
        if basket.status == 'open':
            return basket.draft_invoice
        return basket.invoice


basket_invoice_pdf = BasketInvoicePDFView.as_view()


class BasketValidateView(LoginRequiredMixin, FormView):
    template_name = 'lingo/basket/basket_validate.html'

    def dispatch(self, request, *args, **kwargs):
        if self.request.user.is_authenticated:
            nameid = self.request.user.get_name_id()
            self.basket = get_object_or_404(
                Basket,
                status='open',
                payer_nameid=nameid,
                expiry_at__gt=now(),
            )
            if not self.basket.get_lines():
                raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs['form'] = None
        kwargs['basket'] = self.basket
        return super().get_context_data(**kwargs)

    def post(self, request, *args, **kwargs):
        self.basket.invoice = self.basket.draft_invoice.promote()
        self.basket.status = 'tobepaid'
        self.basket.validated_at = now()
        self.basket.save()
        self.basket.notify('validation')
        return redirect(reverse('lingo-basket-detail'))


basket_validate = BasketValidateView.as_view()


class BasketCancelView(LoginRequiredMixin, FormView):
    template_name = 'lingo/basket/basket_cancel.html'

    def dispatch(self, request, *args, **kwargs):
        if self.request.user.is_authenticated:
            nameid = self.request.user.get_name_id()
            self.basket = get_object_or_404(
                Basket,
                status__in=['open', 'tobepaid'],
                payer_nameid=nameid,
            )
            if not self.basket.get_lines():
                raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs['form'] = None
        kwargs['basket'] = self.basket
        return super().get_context_data(**kwargs)

    def post(self, request, *args, **kwargs):
        self.basket.status = 'cancelled'
        self.basket.cancelled_at = now()
        self.basket.save()
        self.basket.notify('cancel')
        if self.basket.invoice_id:
            self.basket.invoice.cancelled_at = now()
            self.basket.invoice.save()
        return redirect(reverse('lingo-basket-detail'))


basket_cancel = BasketCancelView.as_view()
